/*
 * 文 件 名:  CarbonCalculationService
 * 版    权:  华设设计集团股份有限公司
 * 描    述:  <描述>
 * 修 改 人:  lvwenbin
 * 修改时间:  2025/6/25
 * 跟踪单号:  <跟踪单号>
 * 修改单号:  <修改单号>
 * 修改内容:  <修改内容>
 */
package com.ruoyi.ems.service.analysis;

import com.ruoyi.ems.domain.CaMeterD;
import com.ruoyi.ems.domain.ElecPgSupplyH;
import com.ruoyi.ems.domain.ElecPvSupplyH;
import com.ruoyi.ems.domain.EmissionFactor;
import com.ruoyi.ems.domain.EnergyMeter;
import com.ruoyi.ems.mapper.CaMeterDMapper;
import com.ruoyi.ems.model.QueryMeter;
import com.ruoyi.ems.service.IElecMeterHService;
import com.ruoyi.ems.service.IElecPgSupplyHService;
import com.ruoyi.ems.service.IElecPvSupplyHService;
import com.ruoyi.ems.service.IEmissionFactorService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;

/**
 * 碳计算任务
 * <功能详细描述>
 *
 * @author lvwenbin
 * @version [版本号, 2025/6/25]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
@Service
public class CarbonCalculationService {
    private static final Logger log = LoggerFactory.getLogger(CarbonCalculationService.class);

    private final DateTimeFormatter dateForm = DateTimeFormatter.ofPattern("yyyy-MM-dd");

    @Resource
    private CaMeterDMapper caMeterDMapper;

    @Resource
    private IElecPgSupplyHService pgSupplyHService;

    @Resource
    private IElecPvSupplyHService pvSupplyHService;

    @Resource
    private IEmissionFactorService emissionFactorService;

    /**
     * 计算日碳排量
     * <功能详细描述>
     *
     * @param areaCode 行政区划代码
     * @param date     日期
     */
    public void calculateDailyCarbonEmission(String areaCode, LocalDate date) {
        try {
            CaMeterD caMeterD = new CaMeterD();
            caMeterD.setAreaCode(areaCode);
            caMeterD.setDate(Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));

            // 计算碳排放量（基于电网用电数据）
            ElecPgSupplyH totalElectricity = getDailyElectricityConsumption(areaCode, date);
            Double emission = calculateCarbonEmission(totalElectricity);
            caMeterD.setCaEmissionQuantity(emission);

            // 计算碳汇量（基于光伏产能数据）
            ElecPvSupplyH totalPVProduction = getDailyPVProduction(areaCode, date);
            Double sink = calculateCarbonSink(totalPVProduction);
            caMeterD.setCaSinkQuantity(sink);

            caMeterDMapper.deleteCaMeterDByArea(areaCode, date.format(dateForm));
            caMeterDMapper.insertCaMeterD(caMeterD);
        }
        catch (Exception e) {
            log.error("calculate daily carbon emission error! areaCode:{}, date:{}", areaCode, date.format(dateForm),
                e);
        }
    }

    private ElecPvSupplyH getDailyPVProduction(String areaCode, LocalDate date) {
        QueryMeter queryMeter = new QueryMeter();
        queryMeter.setAreaCode(areaCode);
        queryMeter.setDate(date.format(dateForm));
        List<ElecPvSupplyH> res = pvSupplyHService.calDayPvRange(queryMeter);
        return res.isEmpty() ? null : res.get(0);
    }

    private ElecPgSupplyH getDailyElectricityConsumption(String areaCode, LocalDate date) {
        QueryMeter queryMeter = new QueryMeter();
        queryMeter.setAreaCode(areaCode);
        queryMeter.setDate(date.format(dateForm));
        return pgSupplyHService.calDaySupplyByH(queryMeter);
    }

    // 计算碳排放量
    private Double calculateCarbonEmission(ElecPgSupplyH totalElectricity) {
        EmissionFactor factor = emissionFactorService.selectByRegion("CN", 1);

        if (null == factor || factor.getFactorValue() == null) {
            return null;
        }

        if (totalElectricity == null || totalElectricity.getUseElecQuantity() == null) {
            return null;
        }

        return round(totalElectricity.getUseElecQuantity() * factor.getFactorValue(), 2);
    }

    private Double calculateCarbonSink(ElecPvSupplyH totalPVProduction) {
        EmissionFactor factor = emissionFactorService.selectByRegion("CN", 4);

        if (null == factor || factor.getFactorValue() == null) {
            return null;
        }

        if (totalPVProduction == null || totalPVProduction.getGenElecQuantity() == null) {
            return null;
        }

        return round(totalPVProduction.getGenElecQuantity() * factor.getFactorValue(), 2);
    }

    // 数值四舍五入保留指定小数位
    private double round(double value, int scale) {
        if (scale < 0)
            throw new IllegalArgumentException("scale must be non-negative");
        BigDecimal bd = BigDecimal.valueOf(value);
        bd = bd.setScale(scale, RoundingMode.HALF_UP);
        return bd.doubleValue();
    }
}